#include <stdio.h>
#include <stdint.h>
#include <stdbool.h>
#include "TivaWare/driverlib/sysctl.h"
#include "TivaWare/driverlib/sw_crc.h"
#include "TivaWare/driverlib/crc.h"
#include "TivaWare/inc/hw_types.h"
#include "TivaWare/inc/hw_memmap.h"

#define TEST_FRAME_1_SIZE 14
#define TEST_FRAME_2_SIZE 80

int main(void)
{
    SysCtlPeripheralEnable(SYSCTL_PERIPH_CCM0);
    while(!SysCtlPeripheralReady(SYSCTL_PERIPH_CCM0));

	/*
	 * Test frames, last two bytes are expected crc of the rest of the frame.
	 * Checksums of these were calculated by my PC app, with our own
	 * implementation of CRC16-IBM algorithm, which happens to be almost
	 * exactly the same as TivaWare software implementation (same lookup table etc.).
	 */

    uint8_t test_frame_1[TEST_FRAME_1_SIZE]  = {
    	0x0d, 0xf0, 0x0e, 0x00, 0xf2, 0xff, 0x00, 0x00, 0x07, 0xe0, 0x03, 0x00,
    	0x0E, 0xC5
    };

    uint8_t test_frame_2[TEST_FRAME_2_SIZE]  = {
    	0x0D ,0xF0 ,0x50 ,0x00 ,0xB0, 0xFF ,0x00 ,0x00, 0x07 ,0xE0 ,0x01 ,0x00,
    	0x01 ,0x00 ,0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00 ,0x00,
    	0x00 ,0x00 ,0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00 ,0x00,
    	0x00 ,0x00 ,0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00 ,0x00,
    	0x00 ,0x00 ,0x00 ,0x00 ,0x01, 0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00 ,0x00,
    	0x00 ,0x00 ,0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00, 0x00 ,0x00 ,0x00 ,0x00,
    	0x00 ,0x00 ,0x00 ,0x00 ,0x00, 0x00 ,0xF8 ,0x6F
    };


    /*
     * compute checksum using hw module. Use description form datasheet
     * (www.ti.com/lit/ds/symlink/tm4c1294kcpdt.pdf, p. 946).
     * First, write cfg register:
     */
    HWREG(CCM0_BASE + 0x400) = CRC_CFG_TYPE_P8005 | CRC_CFG_SIZE_8BIT |
    		CRC_CFG_INIT_SEED;

    /*
     * write seed reg to 0:
     */
    HWREG(CCM0_BASE + 0x410) = 0;
	int i;
	/*
	 * Write data in register with input data:
	 */
	for(i = 0; i < TEST_FRAME_1_SIZE - 2; i++)
	{
		HWREG(CCM0_BASE + 0x414) = test_frame_1[i];
	}
	uint32_t hw_crc_1 = HWREG(CCM0_BASE + 0x410); /* I've got 0xB7FE here */

    /*
     * repeat above steps to compute second checksum:
     */
    HWREG(CCM0_BASE + 0x410) = 0;
	for(i = 0; i < TEST_FRAME_2_SIZE - 2; i++)
	{
		HWREG(CCM0_BASE + 0x414) = test_frame_2[i];
	}
	uint32_t hw_crc_2 = HWREG(CCM0_BASE + 0x410); /* hw_crc_2 = 0x41F1 */

    /*
     * use sofware checksum computation from TivWare library:
     */
    uint32_t sw_crc_1 = Crc16(0, test_frame_1, TEST_FRAME_1_SIZE - 2);
    uint32_t sw_crc_2 = Crc16(0, test_frame_2, TEST_FRAME_2_SIZE - 2);

    /*
     * expected checksum is calculated with exactly the same algorithm
     * as used in TivaWare library, so it should be the same.
     */
    uint16_t expected_crc_1 = test_frame_1[TEST_FRAME_1_SIZE - 2];
    expected_crc_1 |= test_frame_1[TEST_FRAME_1_SIZE - 1] << 8;
    uint16_t expected_crc_2 = test_frame_2[TEST_FRAME_2_SIZE - 2];
    expected_crc_2 |= test_frame_2[TEST_FRAME_2_SIZE - 1] << 8;


    /*
     * put breakpoint below to see if calucated checksums match:
     */

	return 0;
}
